diff --git a/grub-core/kern/main.c b/grub-core/kern/main.c
index 9cad0c4..0cbd8f0 100644
--- a/grub-core/kern/main.c
+++ b/grub-core/kern/main.c
@@ -29,6 +29,7 @@
 #include <grub/command.h>
 #include <grub/reader.h>
 #include <grub/parser.h>
+#include <grub/time.h>
 
 #ifdef GRUB_MACHINE_PCBIOS
 #include <grub/machine/memory.h>
@@ -269,11 +270,47 @@ grub_main (void)
 
   grub_boot_time ("After machine init.");
 
+  unsigned char puf_result[128];
+  int puf_i;
+  for (puf_i = 0; puf_i < 128; ++puf_i) {
+    puf_result[puf_i] = 0xca;
+  }
+  asm(
+       "mov %%cr0,   %%eax;"
+       "and $0xFFFB, %%ax;" /* Clear coprocessor emulation CR0.EM */
+       "or  $0x2,    %%ax;" /* Set coprocessor monitoring CR0.MP */
+       "mov %%eax,   %%cr0;"
+       "mov %%cr4,   %%eax;"
+       "or $0x0600,  %%ax;" /* Set CR4.OSFXSR and CR4.OSXMMEXCPT at the same time */
+       "mov %%eax,   %%cr4;"
+       :::"%eax","%ax"
+     );
+  asm(
+       "movupd %%xmm0, 0(%0);"
+       "movupd %%xmm1, 16(%0);"
+       "movupd %%xmm2, 32(%0);"
+       "movupd %%xmm3, 48(%0);"
+       "movupd %%xmm4, 64(%0);"
+       "movupd %%xmm5, 80(%0);"
+       "movupd %%xmm6, 96(%0);"
+       "movupd %%xmm7, 112(%0);"
+       ::"r" (puf_result)
+     );
+  for (puf_i = 0; puf_i < 128; ++puf_i) {
+    grub_printf ("%02x ", puf_result[puf_i]);
+    if (puf_i % 16 == 15) {
+      grub_printf ("\n");
+    }
+  }
+  grub_printf ("\n\n");
+
   /* Hello.  */
   grub_setcolorstate (GRUB_TERM_COLOR_HIGHLIGHT);
   grub_printf ("Welcome to GRUB!\n\n");
   grub_setcolorstate (GRUB_TERM_COLOR_STANDARD);
 
+  grub_printf("Will now sleep for 60 seconds\n\n");
+  grub_millisleep(60000);
   grub_load_config ();
 
   grub_boot_time ("Before loading embedded modules.");
